import {
  Column,
  Entity,
  PrimaryGeneratedColumn,
  CreateDateColumn,
  BeforeInsert,
  BeforeUpdate,
  BaseEntity,
  ManyToMany,
  OneToMany

} from 'typeorm';
import bcrypt from 'bcrypt';
import {nanoid} from 'nanoid';
import {Task} from './Task';

const SALT_WORK_FACTOR= 10;


type userRoleType = "worker" | "director";

interface IUser {
  id:string;
  name: string;
  surname: string;
  email: string;
  displayName: string;
  password:string;
  token: string;
  role: userRoleType;
  createdAt: Date;
  createdTasks:Task[]| null;
}



@Entity({ name: 'User' })
export class User extends BaseEntity implements IUser {
  @PrimaryGeneratedColumn('uuid')
  id!: string
  @Column({ name: 'name', type: 'varchar', length:20,nullable: false })
  name!: string

  @Column({ name: 'surname', type: 'varchar', length:30,nullable: false })
  surname!: string

  @Column({ name: 'displayName', type: 'varchar', length:30,nullable: false })
  displayName!: string

  @Column({ name: 'email', type: 'varchar',length:20, unique: true, nullable: false })
  email!: string

  @Column({ name: 'phone', type: 'varchar',length:10, unique: true, nullable: true})
  phone?: string

  @Column({ name: 'token', type: 'varchar',length:100, unique: true, nullable: false })
  token!: string
  
  @CreateDateColumn({ name: 'created_at', type: Date, default: new Date() })
  createdAt!: Date;
  
  @Column({
    type: "enum",
    enum: ["worker", "director"],
    default: "worker"
  })
  role!: userRoleType
  
  @Column({ type: 'varchar', nullable: false, select:false })
  password!: string

  
  @OneToMany(() => Task, (task: { user: User }) => task.user)
  createdTasks!: Task[];

  @ManyToMany(() => Task, (task: { users: User[] }) =>task.users)
  tasksToMake!: Task[];

  @BeforeInsert()
  protected async beforeInserthashPassword():Promise<void>  {
    const salt = await bcrypt.genSalt(SALT_WORK_FACTOR);
    this.password = await bcrypt.hash(this.password, salt);
  }

  @BeforeUpdate()
  protected async beforeUpdateHashPassword():Promise<void> {
    const salt = await bcrypt.genSalt(SALT_WORK_FACTOR);
    this.password = await bcrypt.hash(this.password, salt);
  }

  public generateToken():void{
    this.token = nanoid()
    return
  }

  static async comparePasswords(
    candidatePassword: string,
    hashedPassword: string
  ):Promise<boolean> {
    return await bcrypt.compare(candidatePassword, hashedPassword);
  }
  

}


























// const UserEntity = new EntitySchema<IUser>({
//     name: "user",
//     columns: {
//         id: {
//             type: "uuid",
//             primary: true,
//             generated: "uuid",
//         },
//         name: {
//           nullable: false,
//             type: String,
//             length: 20,
//         },
//         surname: {
//           nullable: false,
//             type: String,
//             length: 50,
//         },
//         password: {
//           nullable: false,
//             type: String,
//         },
//         email: {
//           nullable: false,
//             type: String,
            
//         },   
//         displayName: {
//           nullable: false,
//             type: String,
//         },
//         role: {
//           type: String,
//           default: "user",
//           enum: ["director","user"]

//         },
//         createdAt: {
//           type: Date,
//           default: Date.now(),
//         },
        // methods?:{
        //   type:String
        // }
    // },
    // relations: {
    //     tasks: {
    //         type: "many-to-many",
    //         target: "category", // CategoryEntity
    //     },
    // },
// })


// UserEntity.methods.checkPassword=function(password:string):Promise<boolean> {
//     return bcrypt.compare(password,this.password);
//   }

// export default UserEntity





















// import { Schema, model } from 'mongoose';
// import bcrypt from 'bcrypt';
// import {nanoid} from 'nanoid';

// const SALT_WORK_FACTOR= 10;

// // 1. Create an interface representing a document in MongoDB.
// interface IUser {
//   name: string;
//   surname: string;
//   email: string;
//   displayName: string;
//   password:string;
//   role: string;
// }

// // 2. Create a Schema corresponding to the document interface.
// const UserSchema = new Schema<IUser>({
//   name: { type: String, required: true },
//   surname: { type: String, required: true },
//   email: { type: String, required: true },
//   displayName: { type: String, required: true },
//   password: { type: String, required: true },
//   role:{type:String,default: 'user',enum:['user','admin']}
// });


// // How does next's type defined?
// UserSchema.pre('save', async function(next:any):Promise<void>{
//   if(!this.isModified('password')) return next();
//   console.log('next', next)
//   const salt = await bcrypt.genSalt(SALT_WORK_FACTOR);
//   const hash = await bcrypt.hash(this.password, salt);
//   this.password = hash;
//   next();
// })

// UserSchema.set('toJSON',{
//   transform:(doc:any, ret:any, options:any)=>{
//       delete ret.password;
//       return ret;
//   }
// })

// UserSchema.methods.checkPassword =function(password:string):Promise<boolean>{
//   return bcrypt.compare(password,this.password);
// }

// UserSchema.methods.generateToken =function(){
//   this.token=nanoid();
// }



// // 3. Create a Model.
// const User = model<IUser>('User', UserSchema);


// export default User;
